/********************************************************************************
 * Copyright (c) 2023 CEA-List
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 ********************************************************************************/

#include <cassert>
#include <chrono>  // std::chrono::milliseconds
#include <numeric> // std::accumulate
#include <thread>  // std::this_thread::sleep_for
#include <vector>

#include "aidge/operator/Mul.hpp"
#include "aidge/utils/Types.h"

#include "aidge/backend/cpu/operator/MulImpl.hpp"
#include "aidge/backend/cpu/operator/MulImpl_forward_kernels.hpp"

Aidge::NbElts_t Aidge::MulImpl_cpu::getNbRequiredProtected(const Aidge::IOIndex_t /*inputIdx*/) const {
    // this implementation can be in-place
    return 0;
}

void Aidge::MulImpl_cpu::forward() {
    assert(mOp.getInput(0) && "missing input #0");
    assert(mOp.getInput(1) && "missing input #1");

    assert(((mOp.getInput(1)->size() == 1) || 
            (mOp.getInput(1)->size() == mOp.getInput(0)->size()) ||
            (mOp.getInput(1)->nbDims() == 1 && mOp.getInput(1)->size() == mOp.getInput(0)->dims()[mOp.getInput(0)->nbDims()-1])
           ) &&
           "input #1 must either be a tensor of size 1, the number of channels of input # or the same size of input #0");

    // Find the correct kernel type
    auto kernelFunc = Registrar<MulImplForward_cpu>::create({
        mOp.getInput(0)->dataType(),
        mOp.getInput(1)->dataType(),
        mOp.getOutput(0)->dataType()});

    // Call kernel
    kernelFunc(std::static_pointer_cast<Tensor>(mOp.getInput(0))->size(),
        std::static_pointer_cast<Tensor>(mOp.getInput(1))->size(),
        mOp.getInput(0)->getImpl()->rawPtr(),
        mOp.getInput(1)->getImpl()->rawPtr(),
        mOp.getOutput(0)->getImpl()->rawPtr());
}
